HEALTHCHECK_ENDPOINT = "/api/system/health"
MESSAGE_BASE_SONARQUBE_USER_TOKEN = "✅ SonarQube User Token is being passed."
MESSAGE_SONARQUBE_NOT_A_USER_TOKEN = "❌ SonarQube User Token is not being passed."
MESSAGE_SONARQUBE_USER_TOKEN_HINT_MESSAGE = "SonarQube User Token starts with 'squ_'. User Token is required to generate the report."
MESSAGE_SONARQUBE_USER_TOKEN_GENERATION_STEPS = "To generate a new SonarQube User Token, please go to your SonarQube account and then navigate to User -> My Account -> Security -> Generate New Token and then select SonarQube User Token."
MESSAGE_SONARQUBE_SERVER_REACHABLE = "✅ SonarQube Server is reachable."
MESSAGE_SONARQUBE_SERVER_UNREACHABLE = "❌ SonarQube Server is unreachable."
MESSAGE_SONARQUBE_SERVER_UNREACHABLE_HINT_MESSAGE = "Please check if the SonarQube Server is reachable and if you have supplied the correct protocol and host. Sometimes VPNs can cause the issue of SonarQube Server being unreachable , so please try to connect to the SonarQube Server without VPN if possible"
MESSAGE_USER_TOKEN_VALIDATED = "✅ SonarQube User Token is valid."
MESSAGE_USER_TOKEN_SOMETHING_WRONG = "❌ Something went wrong while validating the SonarQube User Token. Please try again with a valid token."
MESSAGE_USER_TOKEN_401 = "❌ SonarQube User Token is invalid. We are getting response code 401 which validates that you are Unauthorised to access the SonarQube Server. Please try again with a valid token."
MESSAGE_USER_TOKEN_403 = "❌ SonarQube User Token is invalid. We are getting response code 403 which validates that you are Forbidden to access the SonarQube Server. Please try again with a valid token."
MESSAGE_SUPPORT_APOLOGIES = "🙂‍↕️ Please accept my heartiest apologies if RedCoffee is not working as expected for you. As a sole maintainer of this project, I am trying my best to make it work for everyone. If you are facing any issue, please raise an issue on our GitHub repository - https://github.com/Anubhav9/redcoffee/issues"
MESSAGE_SUPPORT_DIAGNOSE_COMMAND = "❓ Did we check the redcoffee diagnose command? If not, please run the command again and check if the issue persists. If it does, please raise an issue on our GitHub repository"
MESSAGE_SUPPORT_EMAIL = "✍️  Still not sure what's happening? Please shoot an email to anubhavsanyal9@gmail.com"
